setwd("~/Amelie Menut")

library(readxl)
library(ltm)
library(eRm)
library(reshape2)
library(lme4)

rm(list = ls(all.names = TRUE))

#read data word completion experiment in wide format (needed for Rasch analysis); data format = Excel
#put critical data in file mydata

Completion_lmm <- read_excel("Completion_lmm.xlsx")
mydata <- Completion_lmm[ -c(1:2) ]
head(mydata)


#do Rasch analysis 2pl with package ltm()

fit1 <- rasch(mydata)
summary(fit1)

fit2 <- ltm(mydata ~ z1)
summary(fit2)
anova(fit1,fit2)

#plot the items to have an idea how performance differs as a function of item difficulty and participant ability

par(mfrow=c(1,2))
plot(fit1, legend = TRUE, cx = "bottomright", lwd = 3, cex.main = 1.5, cex.lab = 1.3, cex = 1.1)
plot(fit2, legend = TRUE, cx = "bottomright", lwd = 3, cex.main = 1.5, cex.lab = 1.3, cex = 1.1)


#now generate data so that they conform to a Rasch analysis 2pl, with data similar to the ones obtained in the experiment non-transparent
#90 participants: we assume that participants are equally distribution in the range z = -4 to z = +4
#40 items: we assume that the items are equally distributed in the range -2 to +2
#slopes are  fixed according to the analysis of the data of the previous experiment
#data are stored in mydata2
#uses package eRm()

ppar <- runif(90,min = -4, max = 4)
spar <- runif(40,min =-2.5, max = 2.5)
mydata2 <- sim.rasch(ppar, spar)
#include the following lines if you want to see the outcome of the simulation
fit2 <- rasch(mydata2)
par(mfrow=c(1,2))
plot(fit1, legend = TRUE, cx = "bottomright", lwd = 3, cex.main = 1.5, cex.lab = 1.3, cex = 1.1)
plot(fit2, legend = TRUE, cx = "bottomright", lwd = 3, cex.main = 1.5, cex.lab = 1.3, cex = 1.1)

#Put mydata2 in long form so that we will be able to run a mixed effects model
#uses package reshape2()
output1 <- melt(mydata2, id.vars=rownames(mydata2))
#Add participant proficiency because you will need this for the analysis
#Add some noise to proficiency measure, which only correlates r = .7 with the participant ability used in the Rasch model
prof = ppar  + rnorm(90,0,1.7)
cor(prof,ppar)
proficiency <- rep(prof, times=40)
#Add a column saying that these are the items with nonshared suffixes
condition <- rep("nonshared", times = 40*90)
output1 <- cbind(output1,proficiency,condition)
head(output1)


#make new data (40 extra stimuli) with d = 0
#participants remain the same
#because transparent items are supposed to be easier d is subtracted so that the distribution of the shared items is lower
#than the distribution of the nonshared items

d = 0.0
spar <- runif(40,min =-2.5-d, max = 2.5-d)
mydata3 <- sim.rasch(ppar, spar)
#include the following lines if you want to see the outcome of the simulation
#fit3 <- rasch(mydata3)
#plot(fit3, legend = TRUE, cx = "bottomright", lwd = 3, cex.main = 1.5, cex.lab = 1.3, cex = 1.1)

#Put mydata3 in long form so that we will be able to run a mixed effects model
#uses package reshape2()
output2 <- melt(mydata3, id.vars=rownames(mydata3))
#Add a column saying that these are the items with shared suffixes
condition <- rep("shared", times = 40*90)
output2 <- cbind(output2,proficiency,condition)
#make sure that the stimuli in condition 2 are different from those in condition 1 (!!!!!!!!)
output2[,2] <- output2[,2] + 40
head(output2)


#combine the output of the two conditions

output <- rbind(output1,output2)
head(output)
colnames(output) <- c("Part","Stim", "Value", "Proficiency", "Condition")

write.table(output, "simulation.txt", sep=",", row.names = F)

fit3 <- glmer(Value ~ Proficiency*Condition + (Condition|Part) +( 1 | Stim), data=output, family=binomial)
summary(fit3)




#Now we are using the programs above to simulate 400 experiments and look at the distribution of z-values

nSim = 400
zProfi <- numeric(nSim)
zCond <- numeric(nSim)
zInter <- numeric(nSim)

# create progress bar in case it takes a while
pb <- winProgressBar(title = "progress bar", min = 0, max = nSim, width = 300)

for(i in 1:nSim){ #for each simulated experiment
  setWinProgressBar(pb, i, title=paste(round(i/nSim*100, 1), "% done"))
  ppar <- runif(90,min = -4, max = 4)
  prof = ppar  + rnorm(90,0,1.7)
  proficiency <- rep(prof, times=40)
  spar <- runif(40,min =-2.5, max = 2.5)
  mydata2 <- sim.rasch(ppar, spar)
  output1 <- melt(mydata2, id.vars=rownames(mydata2))
  condition <- rep("nonshared", times = 40*90)
  output1 <- cbind(output1,proficiency,condition)
  
  d=1.0
  spar <- runif(40,min = -2.5-d, max = 2.5-d)
  mydata3 <- sim.rasch(ppar, spar)
  output2 <- melt(mydata3, id.vars=rownames(mydata3))
  condition <- rep("shared", times = 40*90)
  output2 <- cbind(output2,proficiency,condition)
  output2[,2] <- output2[,2] + 40
  
  output <- rbind(output1,output2)
  colnames(output) <- c("Part","Stim", "Value", "Proficiency", "Condition")
  fit3 <- glmer(Value ~ Proficiency*Condition + (Condition|Part) +( 1 | Stim), data=output, family=binomial)
  zProfi[i] <- coef(summary(fit3))[2,"z value"]
  zCond[i] <- coef(summary(fit3))[3,"z value"]
  zInter[i] <- coef(summary(fit3))[4,"z value"]
}
close(pb)#close progress bar

par(mfrow=c(3,1))
hist(zProfi)
hist(zCond)
hist(zInter)

zvalues <- cbind(zProfi,zCond,zInter)
write.table(zvalues, "zvaluesd10WC.txt", sep=",", row.names = F)
